// Teensy 3.5 Super CLock 1
// Written by Theron Wierenga, theron@wierengaconsulting.com
// Uses ESP32, using NodeMCU-32S

// This is the ESP32 code

#include <WiFi.h>
#include <HTTPClient.h>
#include "time.h"
#include <WiFiClientSecure.h>
#include <VS1053.h>  //https://github.com/baldram/ESP_VS1053_Library

#define INTERNET 2
#define VS1053_CS    5 
#define VS1053_DCS   33  
#define VS1053_DREQ  35 
#define VOLUME  95 // volume level 0-100
#define STOP_RADIO_PIN 13
#define NEXT_STATION_PIN 14
#define TEST 4

// Place your own WiFi SSID name and password here
const char* ssid       = "Your SSID name";
const char* password   = "password";

//const char* ssid       = "HOME-6A2B-2.4";
//const char* password   = "92A8DE0236D1E1B6";

const char* ntpServer = "pool.ntp.org";
const long  gmtOffset_sec = 3600 * -5;
const int   daylightOffset_sec = 3600;
String h = "12", m = "12", s = "12", y = "2000", mo = "10", d = "10", ttemp = "T12345612345678", zero = "0";
String payload, time_string, weather;
bool debug = false, debug_min = true;
struct tm timeinfo;
  // int tm_sec;
  // int tm_min;
  // int tm_hour;
  // int tm_mday;
  // int tm_mon;
  // int tm_year;
  // int tm_wday;
  // int tm_yday;
  // int tm_isdst;
time_t now;
HTTPClient http;
WiFiClientSecure client_secure;
WiFiClient client;
  
  // Weather
const String endpoint = "http://api.openweathermap.org/data/2.5/weather?id=5004005&units=imperial&APPID=";
const String key =      "22759a049ab498051a6d4c7343bd2d08";
// String temp = "GET /data/2.5/weather?id=" + location + "&units=imperial&APPID=22759a049ab498051a6d4c7343bd2d08 HTTP/1.1";

String tags[13] = {"main", "description", "temp", "pressure", "humidity", "temp_min", "temp_max",
                   "speed", "deg", "gust", "sunrise", "sunset", "name"};
//String mon[12][3] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
String locations[10] = {"5004005", "4175331", "4903279", "5296802", "5650243", "4833387", "5685767", "4160812", "4887398", "5125734"};
String home_location = "4175331";
int loc_index = 1, stock_index = 0;
int num_flag[13] = {0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0}; 
String stock_sym[10] = {"MSFT", "GNTX", "T", "HD", "PFE", "BA", "VZ", "PG", "WMT", "CAT"};
String myInputString = "";
bool myStringComplete = false;

// Few Radio Stations
char *host[5] = {"149.255.59.162", "airspectrum.cdnstream1.com", "comet.shoutca.st", "icecast.omroep.nl", "airspectrum.cdnstream1.com"};
char *path[5] = {"/1", "/1604_128", "/1", "/radio1-bb-mp3", "/1261_192"};
int   httpPort[5] = {8062,8008,8563,80,8000};

//  http://comet.shoutca.st:8563/1
//const char *host = "comet.shoutca.st";
//const char *path = "/1";
//int httpPort = 8563;

uint8_t mp3buff[32];   // vs1053 likes 32 bytes at a time
int radio_station_num = 4;
bool radio_on = false;

VS1053 player(VS1053_CS, VS1053_DCS, VS1053_DREQ);

// ***************************************************************************************
// External Interupt on pin 13 - turns off radio
void IRAM_ATTR stop_radio_Int() 
{
    radio_on = false;
    digitalWrite(TEST, LOW);
}

// ***************************************************************************************
// External Interupt on pin 14 - changes radio station
void IRAM_ATTR next_radio_station_Int() 
{
    radio_station_num += 1;
    if (radio_station_num > 4) radio_station_num = 0;
}

void setup()
{
  if (debug || debug_min)  Serial.begin(115200);
  Serial2.begin(38400);
  delay(3000);
  SPI.begin();
  pinMode(INTERNET, OUTPUT);
  digitalWrite(INTERNET, LOW);
  pinMode(TEST, OUTPUT);
  digitalWrite(TEST, LOW);
  pinMode(STOP_RADIO_PIN, INPUT_PULLUP);
  pinMode(NEXT_STATION_PIN, INPUT_PULLUP);
  pinMode(VS1053_DREQ, INPUT_PULLUP);
  pinMode(VS1053_CS, OUTPUT);
  pinMode(VS1053_DCS, OUTPUT);
  payload.reserve(1000);
  weather.reserve(200);
  //connect to WiFi
  if (debug_min) Serial.printf("Connecting to %s ", ssid);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED)
  {
      delay(2500);
      if (debug_min) Serial.print(".");
  }
  Serial.println(" CONNECTED");
  digitalWrite(INTERNET, HIGH);
  
  //init and get the time
  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);			
 							
  while (!time(nullptr)) {
    Serial.print(".");
    delay(1000);
  }
  //Added to confirme that we are getting a value for time
   Serial.println("Time received");

  initMP3Decoder();

  attachInterrupt(digitalPinToInterrupt(NEXT_STATION_PIN), next_radio_station_Int, FALLING);
  attachInterrupt(digitalPinToInterrupt(STOP_RADIO_PIN), stop_radio_Int, FALLING);

  Serial.println("End of Setup");
}

void loop()
{
  if (WiFi.status() == WL_CONNECTED)
  {  
      mySerialEvent();
      
//      myStringComplete = true;   // remember to remove these
//      myInputString = "Radio";
     
      
      if (myStringComplete)
      {
          if (myInputString == "Time")
          {
              Serial.println("Time");
              getTimeFromInternet();
          }
          if (myInputString == "Home")
          {
              Serial.println("Home");
              getWeatherFromInternet(home_location, myInputString);
          }
          if (myInputString == "Weather")
          {
              Serial.println("Weather");
              getWeatherFromInternet(locations[loc_index], myInputString);
              Serial.println(locations[loc_index]);
              loc_index++;
              if (loc_index > 9) loc_index = 0;
          }
          if (myInputString == "Stock")
          {
              Serial.println("Stock");
              getStocksFromInternet(stock_sym[stock_index]);
              stock_index++;
              if (stock_index > 9) stock_index = 0;
          }
          if (myInputString == "Radio")
          {
              Serial.println("Radio");
              if (client.connect(host[radio_station_num], httpPort[radio_station_num])) 
              {
                  Serial.print("Connected to ");
                  Serial.println(host[radio_station_num]);
    
                  Serial.print("Requesting stream: ");
                  Serial.println(path[radio_station_num]);
        
                  client.print(String("GET ") + path[radio_station_num] + " HTTP/1.1\r\n" +
                                "Host: " + host[radio_station_num] + "\r\n" + 
                                "Connection: close\r\n\r\n");
                  radio_on = true;
                  digitalWrite(TEST, HIGH);
                  while(radio_on)
                  {
                      if(!client.connected())
                      {
                          Serial.println("Reconnecting...");
                          if(client.connect(host[radio_station_num], httpPort[radio_station_num]))
                          {
                            client.print(String("GET ") + path[radio_station_num] + " HTTP/1.1\r\n" +
                                      "Host: " + host[radio_station_num] + "\r\n" + 
                                      "Connection: close\r\n\r\n");
                          }
                      }
  
                      if(client.available() > 0)
                      {
                        uint8_t bytesread = client.read(mp3buff, 32);
                        player.playChunk(mp3buff, bytesread);
                      }
                  }
                  client.stop();
              }
              else
              {
                  Serial.println("Connection failed");
              }
          }
          myInputString = "";
          myStringComplete = false;
       //   Serial.println("Cleared string");
      }
  }
  else
  {
      // Reconnect if signal lost
      digitalWrite(INTERNET, LOW);
      if (debug_min) Serial.println("WiFI lost, restarting...");
      WiFi.disconnect(true);
  //    WiFi.mode(WIFI_OFF);
      WiFi.begin(ssid, password);
      while (WiFi.status() != WL_CONNECTED)
      {
          delay(2500);
          if (debug_min) Serial.print(".");
      }
      digitalWrite(INTERNET, HIGH);
  }
}

void getTimeFromInternet()
{
  configTime(gmtOffset_sec, daylightOffset_sec, ntpServer);
  getLocalTime(&timeinfo);
	now = time(nullptr);
  time_string = ctime(&now);
	if (debug_min) Serial.print(time_string);
        ttemp = "Time";
        h = time_string.substring(11, 13);
        ttemp += h;
        m = time_string.substring(14, 16);
        ttemp += m;
        s = time_string.substring(17, 19);
        ttemp += s;
        y = time_string.substring(20, 24);
        ttemp += y;
        mo = time_string.substring(4, 7);
        // This is ugly
        if (mo == "Jan")  ttemp += "01";
        if (mo == "Feb")  ttemp += "02";
        if (mo == "Mar")  ttemp += "03";
        if (mo == "Apr")  ttemp += "04";
        if (mo == "May")  ttemp += "05";
        if (mo == "Jun")  ttemp += "06";
        if (mo == "Jul")  ttemp += "07";
        if (mo == "Aug")  ttemp += "08";
        if (mo == "Sep")  ttemp += "09";
        if (mo == "Oct")  ttemp += "10";
        if (mo == "Nov")  ttemp += "11";
        if (mo == "Dec")  ttemp += "12";
        d = time_string.substring(8, 10);
        if (d.substring(0,1) == " ") d.setCharAt(0, '0');
        ttemp += d;
        if (debug_min) Serial.println(ttemp);
        
        // Good string looks like this
        //  Time10160420190120
        if (ttemp.length() == 18)
        {
            Serial2.println(ttemp); // This adds \n to end making length 19
            digitalWrite(4, HIGH);
            delay(1000);
            digitalWrite(4, LOW);
        }
}

void getWeatherFromInternet(String location, String isitHome)
{
    const String endpoint = "http://api.openweathermap.org/data/2.5/weather?id=" + location + "&units=imperial&APPID=";
    const String key =      "22759a049ab498051a6d4c7343bd2d08";

  // String temp_location = "GET /data/2.5/weather?id=" + location + "&units=imperial&APPID=22759a049ab498051a6d4c7343bd2d08 HTTP/1.1";
    http.begin(endpoint + key); //Specify the URL
  //  http.begin(temp_location); //Specify the URL
    int httpCode = http.GET();  //Make the request

    if (httpCode > 0)
    {
        // Check for the returning code
        payload = http.getString();
        // Serial.println(httpCode);
        if (debug) 
        {
            Serial.println(payload);
            Serial.print(payload.length());
            Serial.println(" bytes");
        }
        Serial.println(payload);
        weather = parse_it(payload, isitHome);
        Serial2.println(weather);
      }
     else 
     {
          if (debug) Serial.println("Error on HTTP request");
     }
     http.end(); //Free the resources
}

String parse_it(String tag, String isitHome)
{
    int tags_i = 0, index = 0, val1, val2, last_val;
    String tempx = "", last_temp = "";
    String ret_string = isitHome;
    ret_string += ",";
    while (tags_i < 13)
    {
        val1 = tag.indexOf(tags[tags_i], 0);
        if (val1 < 0)
        {
            tags_i += 1;
            continue;
        }
        tempx = tag.substring(val1, val1 + tags[tags_i].length());
        if (debug) Serial.println(tags[tags_i]);
        if (tempx == tags[tags_i])
        {
            if (num_flag[tags_i] == 1)    // It's a numeric value
            {
                val1 = val1 + tags[tags_i].length() + 2;
                index = val1;
                val2 = tag.indexOf("\"", index) - 1;
                tempx = tag.substring(val1, val2);
                if (tempx.endsWith("}"))
                {
                  tempx.remove(tempx.length() - 1, 1);
                }     
                if (debug) Serial.print("            ");
                if (debug) Serial.println(tempx);
                if ((tags[tags_i] == "temp") || (tags[tags_i] == "humidity") || (tags[tags_i] == "speed")|| (tags[tags_i] == "deg"))
                {
                    ret_string += ",";
                    ret_string += tempx;
                    // How long before weather change
//                    if (tags[tags_i] == "temp")
//                    {
//                        if (tempx != last_temp)
//                        {
//                            Serial.print(last_temp);
//                            Serial.print("   -   ");
//                            last_temp = tempx;
//                            Serial.println(last_temp);
//                            Serial.println("New Weather Report ******************************");
//                        }
//                    }
                }
            }
            else                          // It's a String
            {
                index = val1 + tags[tags_i].length() + 1;
                val1 = tag.indexOf("\"", index) + 1;
                index = val1 + 2;
                val2 = tag.indexOf("\"", index);
                tempx = tag.substring(val1, val2);
                if (debug) Serial.print("            ");
                if (debug) Serial.println(tempx);
                if (tags[tags_i] == "description")
                {
                    ret_string += tempx;
                }
                if (tags[tags_i] == "name")
                {
                    ret_string += ",";
                    ret_string += tempx;
                }
            }
            tags_i += 1;
        }
    }
    if (debug_min) Serial.println(ret_string); 
    return ret_string;
    
}

// **********************************************************************
// Get Stock Quotes from Internet
String getStocksFromInternet(String symbol)
{
  // Get stock market data from www.alphavantage.co
  // Click on the following URL and it works in a browser
  // https://www.alphavantage.co/query?function=TIME_SERIES_INTRADAY&symbol=MSFT&interval=1min&outputsize=compact&apikey=HWHEHCWA5TBB7TM
  char  server[] = "api.iextrading.com";
                   //    char server[] = "https://www.alphavantage.co";
                   //    char server[] = "www.google.com";
                   payload = "";
  // String temps = "GET /query?function=TIME_SERIES_INTRADAY&symbol=DJIA&interval=1min&outputsize=compact&apikey=HWHEHCWA5TBB7TM HTTP/1.1";
  // String temps = "GET /search?q=arduino HTTP/1.1";
  //String temps = "GET /1.0/stock/" + symbol + "/ohlc";     // working
  String temps = "GET /1.0//stock/market/batch?symbols=" + symbol + "&types=quote&range=1m&last=1";   // working

  Serial.println("\nStarting connection to server...");
  // if you get a connection, report back via serial:
  if (client_secure.connect(server, 443))
  {
    Serial.println("Connected to SSL server");
    // Make a HTTP request:
    client_secure.println(temps);
    client_secure.println("Host: api.iextrading.com");
    //client_secure.println("Host: www.alphavantage.co");
    client_secure.println("Connection: close");
    client_secure.println();
  }
  delay(500);
  // if there are incoming bytes available
  // from the server, read them and print them:
  // Serial.println(client.available());
  while (client_secure.available())
  {
    char c = client_secure.read();
    payload += c;
  }

  // if the server's disconnected, stop the client:
  if (!client_secure.connected()) {
    Serial.println();
    Serial.println("Disconnecting from server.");
    client_secure.stop();
  }
  client_secure.stop();
//  Serial.print("Payload = ");
//  Serial.println(payload);
  String ps = parse_stock(payload, symbol);
  return ps;

}


String parse_stock(String stock_str, String symbol)
{
    // Find "latestPrice" tag
    int i = stock_str.indexOf("latestPrice");
    int ii = stock_str.indexOf(":", i + 11);
    int iii = stock_str.indexOf(",", ii + 1);
    String pr = stock_str.substring(ii + 1, iii);
    i = stock_str.indexOf("companyName");
    ii = stock_str.indexOf("\"", i + 12);
    iii = stock_str.indexOf("\"", ii + 1);
    String cname = stock_str.substring(ii + 1, iii);
    String stock = "Stock,";
    stock += symbol;
    stock += ",";
    stock += pr;
    stock += ",";
    stock += cname;
    Serial2.println(stock);
    Serial.println(stock);
    return pr; 
  
}

// *********************************************************************************
// Detect and store an incoming serial string
void mySerialEvent()
{
  while ((Serial2.available() > 0) && (myStringComplete == false))
  {
    // get the new byte:
    char inChar = (char)Serial2.read();
    // if the incoming character is a newline, set a flag
    // so the main loop can do something about it:
    // This changed so no \n at end
    if (inChar == '\n')
    {
      myStringComplete = true;
      break;
    }
    // add it to the inputString:
    myInputString += inChar;
  
    delay(2);
  }
  if (myInputString.length() > 0)
  {
      Serial.print("Input string = ");
      Serial.println(myInputString);
    //  Serial.println(myInputString.length());
  }
}

// ********************************************************************************
// Initialize the mp3 decoder
void initMP3Decoder()
{
  player.begin();
  player.switchToMp3Mode(); // optional, some boards require this
  player.setVolume(VOLUME);
}
