// Teensy_3.5_Super_Clock_2

// Written by Theron Wierenga, theron@wierengaconsulting.com
// November 22018 - January 2019
// Maintains time and date with DS3231
// Maintains temperature and humidity with BME280
// Uses RA8875 to interface with LCD display
// Uses VS1053 for MP3 decoding
// Uses PAM8403 for audio amplification

// This is Teensy 3.5 code

#include "RTClib.h"
#include "Wire.h"
#include <SPI.h>
#include <SD.h>
#include "time.h"
#include "Adafruit_GFX.h"     // See www.adafruit.com
#include "Adafruit_RA8875.h"
#include <Adafruit_GFX.h>
#include "SparkFunBME280.h"   // See www.sparkfun.com

// Additonal colors
#define RA8875_DARK_GREEN 0x0400
#define RA8875_ORANGE 0xFB00
#define ALARM 2
#define TEST 33
#define BUFFPIXEL 20

// Pulse these low to get interupt on ESP32
#define STOP_RADIO_PIN 14
#define NEXT_STATION_PIN 17

// Library only supports hardware SPI at this time
// Connect SCLK to UNO or Nano Digital #13 (Hardware SPI clock)
// Connect MISO to UNO or Nano Digital #12 (Hardware SPI MISO)
// Connect MOSI to UNO or Nano Digital #11 (Hardware SPI MOSI)
#define RA8875_INT 3     // GPIO3 on Teensy 3.5 to INT on RA8875
#define RA8875_CS 10     // GPIO4 on Teensy 3.5 to CS on RA8875
#define RA8875_RESET 9   // GPIO5 on Teensy 3.5 to RESET on RA8875

// Create RA8875 driver instance
Adafruit_RA8875 tft = Adafruit_RA8875(RA8875_CS, RA8875_RESET);
uint16_t touchx, touchy;

// Create Real Time Clock instance
RTC_DS3231 rtc;
DateTime now_time;
struct tm timeinfo;

// Create BME280 temperature sensor instance
BME280 mySensor;

// Time server
const char* ntpServer = "pool.ntp.org";
const long  gmtOffset_sec = 3600 * -5;
const int   daylightOffset_sec = 3600;
String time_string, ttemp;

// 7 segment patterns for number 0 - 9
byte num[] = {126,48,109,121,51,91,31,112,127,115};

// Lots of global variables
byte bsecond, stemp, bminute, mtemp, bhour, htemp, ampm, bdayOfWeek, bdayOfMonth, bmonth, b1, b2, b3, b4, last_sec = 0, alarm_ampm = 0, alarm_on = 1;
int last_hour1 = -1, last_min10 = -1, last_min1 = -1, last_sec10 = -1, last_sec1 = -1, current;
int timer_hour10 = 0, timer_hour1 = 0, timer_minute10 = 0, timer_minute1 = 0, timer_second10 = 0, timer_second1 = 0;
int last_timer_hour10 = -1, last_timer_hour1 = -1, last_timer_minute10 = -1, last_timer_minute1 = -1, last_timer_second10 = -1, last_timer_second1 = -1;
int alarm_hour10 = 0, alarm_hour1 = 0, alarm_minute10 = 0, alarm_minute1 = 0, alarm_second10 = 0, alarm_second1 = 0, alarm_hour = 0, alarm_minute = 0;
char daysOfTheWeek[7][10] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
char months[12][10] = {"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"};
char tempfar[8], humid[8], buf[25], itempfar[8], ihumid[8], iwind[8], idesc[35], iname[35] = "Waiting for Weather Report", istart[40] = "Waiting for Internet Connection", Wind[7] = " Wind ";
char htempfar[8], hhumid[8], hbuf[25], hitempfar[8], hihumid[8], hiwind[8], hidesc[35], hiname[35] = "Waiting for Weather Report", lasttempfar[8], last_time[6];
char compass[8][3] = {"N", "NE", "E", "SE", "S", "SW", "W", "NW"};
char cnum[10][2] = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9"};
int tlen, hlen, wlen, wi, hwi, mode = 0,windex = 0;
unsigned int iyear;
unsigned long unixTime, background = 0xFFFF;
unsigned long palette[128];
char but1[8][10] = {"Clock", "Timer", "Weather", "Alarm", "Images", "Stocks", "Mandel", "Radio"};
char but2[2][10] = {"Start", "Exit"};
char but3[2][5] = {"On", "Off"}; 
bool immediate = false, debug = false, debug_min = true, timer_done = true, myStringComplete = false, my_radio_on = false;;
String payload, myInputString;
unsigned long t1, t2;
String locations[10] = {"5004005", "4175331", "4903279", "5296802", "5650243", "4833387", "5685767", "4160812", "4887398", "5125734"};
String home_location = "4175331";
//String stock_sym[10] = {"MSFT", "GNTX", "T", "HD", "PFE", "BA", "VZ", "PG", "WMT", "CAT"};
char stock_symbol[8];
char stock_name[30];
char stock_price[10];
File root;


void setup() 
{
  Serial.begin(115200);
  Serial3.begin(38400);
  pinMode (TEST, OUTPUT);
  pinMode (STOP_RADIO_PIN, OUTPUT);
  pinMode (NEXT_STATION_PIN, OUTPUT);
  digitalWrite(STOP_RADIO_PIN, HIGH);
  digitalWrite(NEXT_STATION_PIN, HIGH);
 
  payload.reserve(4000);
  myInputString.reserve(200);
  // Toggle alarm
  digitalWrite(ALARM, HIGH);
  delay(1000);
  digitalWrite(ALARM, LOW);

  Serial.println("RA8875 start");
  /* Initialise the display using 'RA8875_480x272' or 'RA8875_800x480' */
  if (!tft.begin(RA8875_800x480)) {
    Serial.println("RA8875 Not Found!");
    while (1);
  }
  Serial.println("Found RA8875");

  if (! rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
  }
  
  // For I2C and BME280 enable the following
  mySensor.settings.commInterface = I2C_MODE;
  mySensor.settings.I2CAddress = 0x76;
  mySensor.settings.runMode = 3; //Normal mode
  mySensor.settings.tStandby = 0;
  mySensor.settings.filter = 0;
  mySensor.settings.tempOverSample = 1;
  mySensor.settings.pressOverSample = 1;
  mySensor.settings.humidOverSample = 1;
  mySensor.begin();
  mySensor.calibration.dig_T1;
  mySensor.calibration.dig_T2;
  mySensor.calibration.dig_T3;
  mySensor.calibration.dig_P1;
  mySensor.calibration.dig_P2;
  mySensor.calibration.dig_P3;
  mySensor.calibration.dig_P4;
  mySensor.calibration.dig_P5;
  mySensor.calibration.dig_P6;
  mySensor.calibration.dig_P7;
  mySensor.calibration.dig_P8;
  mySensor.calibration.dig_P9;
  mySensor.calibration.dig_H1;
  mySensor.calibration.dig_H2;
  mySensor.calibration.dig_H3;
  mySensor.calibration.dig_H4;
  mySensor.calibration.dig_H5;
  mySensor.calibration.dig_H6;
  Serial.print("BME280 temp = ");
  Serial.println(mySensor.readTempF());

  if (!SD.begin(254)) 
  {
    Serial.println("initialization failed!");
  }
  
  // Turn on the display and blank out the screen with backcolor
  tft.displayOn(true);
  tft.GPIOX(true);      // Enable TFT - display enable tied to GPIOX
  tft.PWM1config(true, RA8875_PWM_CLK_DIV1024); // PWM output for backlight
  tft.PWM1out(255);
  tft.fillScreen(background);
  tft.textMode();
  tft.textTransparent(RA8875_BLUE);
  tft.textEnlarge(2);
  tft.textSetCursor(20, 200);
  delay(100);
  tft.textWrite(istart, 40);

  tft.graphicsMode();
  tft.fillScreen(background);
  tft.touchEnable(true);

  now_time = rtc.now();
  draw_date_temp_hum(true);
  draw_weather(50, 370, RA8875_BLACK, "Home");
  
  // Run the following with your current date and time settings
  // The comment out this line and updload the program to Arduino a second time
  //                     year mon day hr min sec
  // rtc.adjust(DateTime(2018, 12, 1, 10, 6, 0));

// tone(7, 440);


}

void loop() 
{
      Serial.print("MODE top =  ");
      Serial.println(mode); 
      switch(mode)
      {
          case 0: // Get the current time
                  // This delays things until the second changes
                  while (now_time.second() == last_sec)
                  {
                     now_time = rtc.now();
                  }   
                  t1 =  millis();
                  last_sec = now_time.second();
                  Serial.println(last_sec);
                  bhour = now_time.hour();
                  // Make adjustment for a 12 hour clock
                  ampm = 0;
                  // Check if 12 AM
                  if (bhour == 0) bhour = 12;
                  
                  // Check if it's 12 PM
                  if (bhour == 12) ampm = 1;
                  
                  // Check if it's PM
                  if (bhour > 12)
                  {
                    bhour -= 12;
                    ampm = 1;
                  }
                  // Draw the time in large numerals
                  draw_time(false);

                  // Second 0, Draw date and temp, humidity and weather
                  if (last_sec == 0)
                  {
                    draw_date_temp_hum(false);
                    draw_weather(50, 370, RA8875_BLACK, "Home");
                  }

                  // Check the alarm
                  if (alarm_on)
                  {
                      // Second 1, Checking the alarm
                      if (last_sec == 1)
                      {
                          // Is it alarm time?
//                          Serial.print("Alarm  ");
//                          Serial.print(bhour);
//                          Serial.print("  ");
//                          Serial.println(now_time.minute());
                          if ((alarm_hour == bhour) && (alarm_minute == now_time.minute()) && (alarm_ampm == ampm))
                          {
                                // Turn on the radio for an alarm wake up
                                Serial3.println("Radio\n");
                                my_radio_on = true;
                                // Also put tone on the alarm line
                                tone(ALARM, 440, 4000);
                          }
                      }
                  }

                  // Second 2, Request home weather once a minute
                  if (last_sec == 2)
                  {
                      Serial3.println("Home\n");
                      Serial.println("Home request sent");
                  }

                  // Second 30, Request the time each minute or once an hour
                  // if ((last_sec == 30) && (now_time.minute() == 0))
                  if (last_sec == 30)
                  {
                      Serial3.println("Time\n");
                      Serial.println("Time request sent");
                  }

                  break;
          case 1: // Timer
                  if (!timer_done)
                  {
                      while (now_time.second() == last_sec)
                      {
                         now_time = rtc.now();
                      }
                      last_sec = now_time.second();
                      timer_second1 -= 1;
                      if (timer_second1 < 0)
                      {
                          timer_second1 = 9;
                          timer_second10 -= 1;
                          if (timer_second10 < 0)
                          {
                              timer_second10 = 5;
                              timer_minute1 -= 1;
                              if (timer_minute1 < 0)
                              {
                                  timer_minute1 = 9;
                                  timer_minute10 -= 1;
                                  if (timer_minute10 < 0)
                                  {
                                      timer_minute10 = 5;
                                      timer_hour1 -= 1;
                                      if (timer_hour1 < 0)
                                      {
                                          timer_hour1 = 9;
                                          timer_hour10 -= 1;
                                      }
                                  }      
                              }
                          }   
                      } 
                      draw_timer_time(55, 160, RA8875_RED);      
                      if ((timer_second1 == 0) &&
                          (timer_second10 == 0) &&
                          (timer_minute1 == 0) &&
                          (timer_minute10 == 0) &&
                          (timer_hour1 == 0) &&
                          (timer_hour10 == 0))
                       {
                            timer_done = true;
                            // Turn on the radio when timer ends
                            Serial3.println("Radio\n");
                            my_radio_on = true;
                            // Also put tone on the alarm line
                            tone(ALARM, 440, 4000);
                            tft.fillScreen(background);
                       }
                  }
                  else
                  {
                    draw_timer_time_sm(100, 70, RA8875_RED);
                     // Draw keyboard
                    draw_keyboard(460, 70, RA8875_RED);
                    draw_timer_buttons(150, 180, RA8875_BLUE);
                    get_alarm_timer_time(460, 70, RA8875_RED, false);
                    Serial.print("Mode after timer exit = ");
                    Serial.println(mode);
                    delay(500);
                  }
                  break;
          case 2: // Weather
                  tft.fillScreen(background);
                  for (int i = 0; i < 5; i++)
                  {
                      Serial3.println("Weather\n");
                      while(true)
                      {
                        mySerialEvent();
                        if (myStringComplete)
                        {
                            check_incomming(); 
                            break;
                        }
                      }
                      delay(1000);
                      if (i%2 == 0)
                      {
                          draw_weather(20, 10 + i * 95, RA8875_BLUE, "Weather");
                      }
                      else
                      {
                          draw_weather(20, 10 + i * 95, RA8875_RED, "Weather");
                      }
                      if (get_touch(&touchx, &touchy)) {mode = 0; break;}
                  }

                  // This is the delay between screens
                  for (int i = 0; i < 600; i++)
                  {
                      delay(10); 
                      if (mode == 0)  break;
                      if (get_touch(&touchx, &touchy)) {mode = 0; break;}
                     
                  }
                  tft.fillScreen(background);
                  for (int i = 5; i < 10; i++)
                  {
                      if (mode == 0)  break;
                      Serial3.println("Weather\n");
                      while(true)
                      {
                        mySerialEvent();
                        if (myStringComplete)
                        {
                            check_incomming(); 
                            break;
                        }
                      }
                      delay(1000);
                      if (i%2 == 0)
                      {
                          draw_weather(20, 10 + (i - 5) * 95, RA8875_RED, "Weather");
                      }
                      else
                      {
                          draw_weather(20, 10 + (i - 5) * 95, RA8875_DARK_GREEN, "Weather");
                      }
                      if (get_touch(&touchx, &touchy)) {mode = 0; break;}
                  }

                  for (int i = 0; i < 600; i++)
                  {
                      delay(10); 
                      if (mode == 0)  break;
                      if (get_touch(&touchx, &touchy)) {mode = 0; break;}
                     
                  }
                  if (mode == 0)
                  {
                      tft.fillScreen(background);  
                  //    getWeatherFromInternet(home_location);
                      draw_time(true);
                      draw_date_temp_hum(true);
                      draw_weather(50, 370, RA8875_BLACK, "Home");
                  }
                  break;
          case 3: // Alarm
                  draw_alarm_time_sm(100, 70, RA8875_DARK_GREEN);
                  draw_keyboard(460, 70, RA8875_DARK_GREEN);
                  draw_alarm_buttons(150, 180, RA8875_RED);
                  get_alarm_timer_time(460, 70, RA8875_RED, true);
                  break;
          case 4: // Bitmaps from SD mcard
                  // These two lines draw bitmap images from the sd card 
                  root = SD.open("/");
                  printDirectory(root);
                    
                  if (get_touch(&touchx, &touchy))
                  {
                      tft.fillScreen(background);
                      draw_time(true);
                      draw_date_temp_hum(true);
                      draw_weather(50, 370, RA8875_BLACK, "Home");
                      mode = 0;
                      break;
                  }
                   break;
          case 5: // Stock Quotes
                  tft.fillScreen(background);  
                  for (int i = 0; i < 10; i++)
                  {
                     Serial3.println("Stock\n");
                     while(true)
                      {
                        mySerialEvent();
                        if (myStringComplete)
                        {
                            check_incomming(); 
                            break;
                        }
                      }
                    tft.textMode();
                    tft.textEnlarge(1);
                    tft.textTransparent(RA8875_BLUE);
                    for (int j = 0; j < 2; j++)
                    {
                        tft.textSetCursor(30 + j, 10 + j + i * 45);
                        tft.textWrite(stock_symbol, 8);
                        tft.textWrite(stock_price, 10);
                        tft.textWrite(stock_name, 30);
                    }
                    delay(3000);
                    if (get_touch(&touchx, &touchy))
                    {
                        tft.fillScreen(background);
                        draw_time(true);
                        draw_date_temp_hum(true);
                        draw_weather(50, 370, RA8875_BLACK, "Home");
                        mode = 0;
                        break;
                    }
                  }
                  break;
           case 6:// Mandelbrot images
                  // These two lines create Mandelbrot images
                  create_pallette();
                  mandelbrot();
                  break;
           case 7:radio_menu(320, 70, RA8875_BLUE);
                  break;
      }

      check_incomming(); 
      
      if (get_touch(&touchx, &touchy))
      {
        menu();
      }

      t2 = millis();
      Serial.print("Loop time = ");
      Serial.println(t2 - t1);
}

// **********************************************************************
// Has touch occured?
bool get_touch(uint16_t *x, uint16_t *y)
{
    float xScale = 1024.0F/tft.width();
    float yScale = 1024.0F/tft.height();
    uint16_t tx, ty;
    if(!tft.touched()) {return false;}   // Do we have a touch
    //Serial.println("Inside touch");
    while(tft.touched())
    {
         tft.touchRead(&tx, &ty);
         delay(100);
    }
    *x = tx/xScale;                     // We read the last touch
    *y = ty/yScale;
     Serial.print("Touched  ");
     Serial.print(tx/xScale);
     Serial.print("  ");
     Serial.println(ty/yScale);
     delay(50);
     return true;
}

// **********************************************************************
// Six button menu
void menu()
{
  //    int x[6] = {200, 400, 600, 200, 400, 600};
  //    int y[6] = {140, 140, 140, 300, 300, 300};
      int x[8] = {100, 300, 500, 700, 100, 300, 500, 700};
      int y[8] = {140, 140, 140, 140, 300, 300, 300, 300};
      timer_done = true;
      bool flag = false;
      tft.fillScreen(background);
      // Draw 4 circle for buttons
      for (int i = 0; i < 8; i++)
      {
          for (int k = 0; k < 4; k++)
          {
              tft.drawCircle(x[i], y[i], 40 - k, RA8875_BLUE);
          }
      }
      tft.textMode();
      tft.textTransparent(RA8875_RED);
      tft.textEnlarge(1);
      // Draw button names
      for (int ii = 0; ii < 8; ii++)
      {
          tft.textSetCursor(x[ii] - 40, y[ii] + 40);
          tft.textWrite(but1[ii]);
          tft.textSetCursor(x[ii] - 39, y[ii] + 41);
          tft.textWrite(but1[ii]);
      }

      while (!flag)
      {
          // Now wait for touch
          while(!get_touch(&touchx, &touchy)){}
          
          tft.graphicsMode();
          for (int j = 0; j < 8; j++)
          {
              int xd = touchx - x[j];
              int yd = touchy - y[j];
//              Serial.print(xd);
//              Serial.print("  xd  yd  ");
//              Serial.println(yd);
              if ((abs(xd) < 40) && (abs(yd) < 40))
              {
                  tft.fillCircle(x[j], y[j], 40, RA8875_GREEN);
                  mode = j;
                  Serial.print("MODE SET in menu =  ");
                  Serial.println(mode);
                  delay(200);
                  flag = true;
                  break;
              }
          }
      }
      tft.fillScreen(background);
      if (mode == 0)
      {
        draw_time(true);
        draw_date_temp_hum(true);
        draw_weather(50, 350, RA8875_BLACK, "Home");
      }
      return;
}

// **********************************************************************
// Draw a white rectangle to erase the presious number
void draw_blank(int x, int y, unsigned long color)
{
    tft.fillRect(x, y, 80, 160, color);
}

// **********************************************************************
// Draw a large 7 segment number on the LCD
// Takes x,y location of the upper left corner, the 7 segment pattern 
// of the numbers 0 - 9, stored in a single byte, and the color
void draw_num(int x, int y, byte num, unsigned long color)
{
    int m = 0; 
    for (int k = 0; k < 8; k++)
    {
        // Shift through the byte to get which segments to draw
        m = num & (1<<k);
        if (m > 0)   // Draw this segment  byte = DP,a,b,c,d,e,f,g
        {
            switch (m)
            {
             case 64: // Segment a
                      tft.fillRect(x, y, 80, 20, color);
                      break;
             case 32: // Segment b
                      tft.fillRect(x + 60, y, 20, 90, color);
                      break;
             case 16: // Segment c
                      tft.fillRect(x + 60, y + 70, 20, 90, color);
                      break;
             case 8:  // Segment d
                      tft.fillRect(x, y + 140, 80, 20, color);
                      break;
             case 4:  // Segment e
                      tft.fillRect(x, y + 70, 20, 90, color);
                      break;
             case 2:  // Segment f
                      tft.fillRect(x, y, 20, 90, color);
                      break;
             case 1:  // Segment g
                      tft.fillRect(x, y + 70, 80, 20, color);
                      break;
             case 128: // Deciaml point
                      tft.fillCircle(x + 89, y + 155, 5, color);
                      break;        
            }
        }  
    }
}


// **********************************************************************
// Draws the time across the top of the display, goes down to y = 190
void draw_time(bool imm)
{
      // Now draw the time digits on the LCD display
      // Before drawing each digit we draw a blank white
      // rectangle to erase the previous digit
      // Can we skip a leading 0 on hours
      if (bhour > 9)
      {
          draw_num(10, 30, num[1], RA8875_BLUE);
      }
      else
      {
          draw_blank(10, 30, background);
      }

      // Draw the remaining time digits
      // We only draw the digits if they have changed
      // Hours units
      current = bhour%10;
      if ((last_hour1 != current) || imm)
      {
          draw_blank(110, 30, background);
          draw_num(110, 30, num[current], RA8875_BLUE);
          last_hour1 = current;
      }
      
      // Minutes tens
      current = now_time.minute()/10;
      if ((last_min10 != current) || imm) 
      {         
          draw_blank(310, 30, background);
          draw_num(310, 30, num[current], RA8875_BLUE);
          last_min10 = current;
      }
      
      // Minutes units
      current = now_time.minute()%10;
      if ((last_min1 != current) || imm)
      {
          draw_blank(410, 30, background);
          draw_num(410, 30, num[current], RA8875_BLUE);
          last_min1 = current;
      }
      
      // Seconds tens
      current = now_time.second()/10;
      if ((last_sec10 != current) || imm)
      {
          draw_blank(610, 30, background);
          draw_num(610, 30, num[current], RA8875_BLUE);
          last_sec10 = current;
      }
      // Seconds units
      current = now_time.second()%10;
      if ((last_sec1 != current) || imm)
      {
          draw_blank(710, 30, background);
          draw_num(710, 30, num[current], RA8875_BLUE);
          last_sec1 = current;
      }
    // Draw sun/moon for am/pm
    if (ampm == 0)
    {
        // Dark yellow
        tft.fillCircle(250, 70, 14, 0xFFA0);
        tft.fillCircle(250, 150, 14, 0xFFA0);
        tft.fillCircle(550, 70, 14, 0xFFA0);
        tft.fillCircle(550, 150, 14, 0xFFA0);
    }
    else
    {
        // Dark gray
        tft.fillCircle(250, 70, 14, 0x8410);
        tft.fillCircle(250, 150, 14, 0x8410);
        tft.fillCircle(550, 70, 14, 0x8410);
        tft.fillCircle(550, 150, 14, 0x8410);
    }
}

// **********************************************************************
// Draw the Date
void draw_date_temp_hum(bool imm)
{
    // Draw the date, temerature and humidity once a minute
    // This gets messy as the RA8875 requires character arrays
    // as the parameters in its funtion calls
    if ((now_time.second() < 3) || imm)
    {
        char comma[3] = ", ";
        char space[2] = " ";
        const char deg = 176;
        char F[2] = "F";
        char P[3] = "%H";
        String sday, smonth, syear;
        // Read temperature and humidity from the BME280
        String temperature = String(mySensor.readTempF());
        String humidity = String(mySensor.readFloatHumidity());
        // Switch to char arrays
        temperature.toCharArray(tempfar, 8);
        humidity.toCharArray(humid, 8);
        // Clear the bottom of the LCD display
        tft.fillRect(20, 220, 755, 250, RA8875_GREEN);
        // Switch to text mode
        tft.textMode();
        tft.textTransparent(RA8875_RED);
       // tft.textColor(RA8875_RED, RA8875_WHITE);
        // Specify enlarged text
        tft.textEnlarge(3);

         // The next two for loops run twice just to make the text bolder
        for (int w = 0; w < 2; w++)
        {
            // Set the cursor position
            tft.textSetCursor(30 + w, 220 + w);
            // Write day of the week
            tft.textWrite(daysOfTheWeek[now_time.dayOfTheWeek()], 9);
            tft.textWrite(space, 1);
            // Write local temperature
            tft.textWrite(tempfar, 5);
          //  tft.textWrite(&deg, 1);
            tft.textWrite(F, 1);
            tft.textWrite(space, 1);
            // Write local humidity
            tft.textWrite(humid, 4);
            tft.textWrite(P, 2);
            // Reset the cursor position
            tft.textSetCursor(30 + w, 280 + w);
            // Write the month
            smonth = months[now_time.month() - 1];
            smonth.toCharArray(buf, smonth.length() + 1);
            tft.textWrite(buf);
            tft.textWrite(space, 1);
            // Write the day
            sday = String(now_time.day());
            sday.toCharArray(buf, 3);
            tft.textWrite(buf);
            tft.textWrite(comma, 2);
            // Write the year
            syear = String(now_time.year());
            syear.toCharArray(buf, 5);
            tft.textWrite(buf);
        }
        // Return to graphics mode
        tft.graphicsMode();
    }
}

// **********************************************************************
// Draw Weather Info
void draw_weather(int x, int y, unsigned long color, String type)
{
    char comma[3] = ", ";
    char at[3] = "at";
    char space[2] = " ";
    const char deg = 176;
    char F = 'F';
    char P[3] = "%H";
    // Reduce text size, reset cursor and change color
    tft.textMode();
    tft.textEnlarge(1);
    tft.textTransparent(color);
   
    for (int w = 0; w < 2; w++)
    {
        tft.textSetCursor(x + w, y + w);
        // Write weather location name
        if (type == "Home")
        {
            tft.textWrite(hiname);
        }
        else
        {
             tft.textWrite(iname);
        }
        
        // No weather station has these first three letters
        // If the name hasn't changed it will write the waiting message
        if (!((hiname[0] == 'W') && (hiname[1] == 'a') && (hiname[2] == 'i')))
        {
            tft.textWrite(comma, 2);
            // Write weather description
            if (type == "Home")
            {
                tft.textWrite(hidesc);
            }
            else
            {
                tft.textWrite(idesc);
            }
            // Reset cursor
            tft.textSetCursor(x + w, y + 40 + w);
            // Write temperature
            if (type == "Home")
            {
                tft.textWrite(hitempfar, tlen);
            }
            else
            {
                tft.textWrite(itempfar, tlen);
            }
        //    tft.textWrite(&deg, 1);
            tft.textWrite(&F, 1);
            tft.textWrite(space, 1);
            // Write humidity
            if (type == "Home")
            {
                tft.textWrite(hihumid, hlen);
            }
            else
            {
                tft.textWrite(ihumid, hlen);
            }
            tft.textWrite(P, 2);
            // Write wind speed
            tft.textWrite(Wind,6);
            if (type == "Home")
            {
                tft.textWrite(hiwind, wlen);
            }
            else
            {
                tft.textWrite(iwind, wlen);
            }
            tft.textWrite(space, 1);
            // Write wind direction
            if (type == "Home")
            {
                tft.textWrite(compass[hwi], 2);
            }
            else
            {
                tft.textWrite(compass[wi], 2);
            }
            tft.textWrite(space, 1);
            tft.textWrite(space, 1);
            if (type == "Home")
            {
               tft.textTransparent(RA8875_BLUE);
               tft.textWrite(at, 2);
               tft.textWrite(space, 1);
               tft.textWrite(last_time, 5);
               tft.textTransparent(color);
            }
        }
    }
    // Return to graphics mode
    tft.graphicsMode();
}

// **********************************************************************
// Draw large timer time
void draw_timer_time(int x, int y, unsigned long color)
{
      if (last_timer_hour10 != timer_hour10)
      {
          draw_blank(x, y, background);
          draw_num(x, y, num[timer_hour10], color);
          last_timer_hour10 = timer_hour10;
      }

      if (last_timer_hour1 != timer_hour1)
      {
          draw_blank(x + 100, y, background);
          draw_num(x + 100, y, num[timer_hour1], color);
          last_timer_hour1 = timer_hour1;
      }

      tft.fillCircle(x + 215, y + 40, 14, color);
      tft.fillCircle(x + 215, y + 120, 14, color);

      if (last_timer_minute10 != timer_minute10)
      {
          draw_blank(x + 250, y, background);
          draw_num(x + 250, y, num[timer_minute10], color);
          last_timer_minute10 = timer_minute10;
      }

      if (last_timer_minute1 != timer_minute1)
      {
          draw_blank(x + 350, y, background);
          draw_num(x + 350, y, num[timer_minute1], color);
          last_timer_minute1 = timer_minute1;
      }

      tft.fillCircle(x + 465, y + 40, 14, color);
      tft.fillCircle(x + 465, y + 120, 14, color);

      if (last_timer_second10 != timer_second10)
      {
          draw_blank(x + 500, y, background);
          draw_num(x + 500, y, num[timer_second10], color);
          last_timer_second10 = timer_second10;
      }

      if (last_timer_second1 != timer_second1)
      {
          draw_blank(x + 600, y, background);
          draw_num(x + 600, y, num[timer_second1], color);
          last_timer_second1 = timer_second1;
      }
}

// **********************************************************************
// Draw small timer time
void draw_timer_time_sm(int x, int y, unsigned long color)
{
  char co[2] = ":";
  tft.textMode();
  tft.textTransparent(color);
  tft.textEnlarge(3);
  for (int i = 0; i < 2; i++)
  {
      tft.textSetCursor(x + i, y + i);
      tft.textWrite(cnum[timer_hour10], 1);
      tft.textWrite(cnum[timer_hour1], 1);
      tft.textWrite(co, 1);
      tft.textWrite(cnum[timer_minute10], 1);
      tft.textWrite(cnum[timer_minute1], 1);
      tft.textWrite(co, 1);
      tft.textWrite(cnum[timer_second10], 1);
      tft.textWrite(cnum[timer_second1], 1);
  }
}

// **********************************************************************
// Draw small alarm time
void draw_alarm_time_sm(int x, int y, unsigned long color)
{
  char co[2] = ":";
  tft.textMode();
  tft.textTransparent(color);
  tft.textEnlarge(3);
  for (int i = 0; i < 2; i++)
  {
      tft.textSetCursor(x + i, y + i);
      tft.textWrite(cnum[alarm_hour10], 1);
      tft.textWrite(cnum[alarm_hour1], 1);
      tft.textWrite(co, 1);
      tft.textWrite(cnum[alarm_minute10], 1);
      tft.textWrite(cnum[alarm_minute1], 1);
      tft.textWrite(co, 1);
      tft.textWrite(cnum[alarm_second10], 1);
      tft.textWrite(cnum[alarm_second1], 1);
  }
}

// **********************************************************************
// Draw Keyboard
void draw_keyboard(int x, int y, unsigned long color)
{
  char cl[3] = "Cl";
  char bs[2] = "<";
   
  // Draw the 3x4 grid
  for (int i = 0; i < 5; i++)
  {
       tft.fillRect(x, y + i * 80, 240, 10, color);
       if (i < 4)
       tft.fillRect(x + i * 80, y, 10, 330, color);
  }
  tft.textMode();
  tft.textTransparent(color);
  // Specify enlarged text
  tft.textEnlarge(3);
 
  // Draw numbers 1 - 9
  for (int i = 1; i < 10; i++)
  {
      if (i < 4) tft.textSetCursor(x + 30 + (i - 1) * 80, y + 10);
      if ((i > 3) && (i < 7)) tft.textSetCursor(x + 30 + (i - 4) * 80, y + 90);
      if (i > 6) tft.textSetCursor(x + 30 + (i - 7) * 80, y + 170);
      tft.textWrite(cnum[i], 1);
  }
  // Draw bottom row with chars
  tft.textSetCursor(x + 15, y + 250);
  tft.textWrite(cl, 2);
  tft.textSetCursor(x + 110, y + 250);
  tft.textWrite(cnum[0], 1);
  tft.textSetCursor(x + 190, y + 250);
  tft.textWrite(bs, 1);

}

// **********************************************************************
// Draw Timer Buttons
void draw_timer_buttons(int x, int y, unsigned long color)
{
  char tm[6] = "Timer";

  tft.textEnlarge(3);
  tft.textTransparent(color);
  tft.textSetCursor(x, y);
  tft.textWrite(tm, 5);
  // Write labels on larger buttons
  tft.textEnlarge(1);
  // Draw button names
  for (int ii = 0; ii < 2; ii++)
  {
      tft.textSetCursor(x - 20 + ii * 135, y + 205);
      tft.textWrite(but2[ii]);
      tft.textSetCursor(x - 19 + ii * 135, y + 205);
      tft.textWrite(but2[ii]);
  }
  // Draw circles for circular buttons
  tft.graphicsMode();
  for (int i = 0; i < 2; i++)
  {
      for (int k = 0; k < 4; k++)
      {
          tft.drawCircle(x + 20 + i * 125, y + 160, 40 - k, color);
      }
  }  
}

// **********************************************************************
// Draw Alarm Buttons
void draw_alarm_buttons(int x, int y, unsigned long color)
{
  char tm[6] = "Alarm";
  char b[3][7] = {"On Off", "AM PM", " Exit"};

  tft.textEnlarge(3);
  tft.textTransparent(color);
  tft.textSetCursor(x, y);
  tft.textWrite(tm, 5);
  // Write labels on larger buttons
  tft.textEnlarge(1);
  // Draw button names
  for (int ii = 0; ii < 3; ii++)
  {
      tft.textSetCursor(x - 85 + ii * 125, y + 205);
      tft.textWrite(b[ii], 6);
      tft.textSetCursor(x - 84 + ii * 125, y + 205);
      tft.textWrite(b[ii], 6);
  }
  // Draw circles for circular buttons
  tft.graphicsMode();
  for (int i = 0; i < 3; i++)
  {
      for (int k = 0; k < 4; k++)
      {
          tft.drawCircle(x - 42 + i * 125, y + 160, 40 - k, color);
      }
  }  
}


// **********************************************************************
// Enter alarm or Timer time
void get_alarm_timer_time(int x, int y, unsigned long color, bool alarm)
{
  int num;
  char am_pm[2][3] = {"AM", "PM"};
  char off_on[2][4] = {"Off", "On "};
  if (alarm)
  {
      tft.graphicsMode();
      tft.fillRect(x - 90, y + 140, 50, 30, background);
      tft.fillRect(x - 90, y + 30, 30, 30, background);
      tft.textMode();
      tft.textTransparent(color);
      tft.textEnlarge(1);
      tft.textSetCursor(x - 90, y + 140);
      tft.textWrite(off_on[alarm_on], 3);
      tft.textSetCursor(x - 90, y + 30);
      tft.textWrite(am_pm[alarm_ampm], 2);
  }
  // Wait for a touch
  while(true)
  {
      while(!get_touch(&touchx, &touchy)){};
      num = -1;
      delay(100);
      // Is there a touch on the keyboard?
      for (int i = 1; i < 13; i++)
      {
          if (i < 4) 
          {
            if ((abs((x + 30 + (i - 1) * 80) - touchx) < 40) &&  (abs((y + 80) - touchy) < 35))
            {
              num = i;
              break;
            }
          }
          if ((i > 3) && (i < 7))
          {
            if ((abs((x + 30 + (i - 4) * 80) - touchx) < 40) && (abs((y + 125) - touchy) < 35))
            {
              num = i;
              break;
            }
          }
          if ((i > 6) && (i < 10))
          {
            if ((abs((x + 30 + (i - 7) * 80) - touchx) < 40) && (abs((y + 205) - touchy) < 35))
            {
              num = i;
              break;
            }
          }
          
          if (i > 9)
          {
            if ((abs((x + 30 + (i - 10) * 80) - touchx) < 40) && (abs((y + 285) - touchy) < 35))
            {
              num = i;
              break;
            }
          }
      }

      // Is it a zero?     
      if (num == 11) num = 0;
      // Is it a number 1 - 9?
      if ((num >= 0) && (num < 10))
      {
          Serial.println("Number entered");
          if (alarm)
          {
              alarm_hour10 = alarm_hour1;
              alarm_hour1 = alarm_minute10;
              alarm_minute10 = alarm_minute1;
              alarm_minute1 = alarm_second10;
              alarm_second10 = alarm_second1;
              alarm_second1 = num;
              tft.fillRect(x - 360, y, 265, 65, background);
              draw_alarm_time_sm(x - 360, y, color);
          }
          else
          {
              timer_hour10 = timer_hour1;
              timer_hour1 = timer_minute10;
              timer_minute10 = timer_minute1;
              timer_minute1 = timer_second10;
              timer_second10 = timer_second1;
              timer_second1 = num;
              tft.fillRect(x - 360, y, 265, 65, background);
              draw_timer_time_sm(x - 360, y, color);
          }
      }
      // Is it a clear all
      if (num == 10)
      {
          Serial.println("Clear all");
          if (alarm)
          {
              alarm_hour10 = 0;
              alarm_hour1 = 0;
              alarm_minute10 = 0;
              alarm_minute1 = 0;
              alarm_second10 = 0;
              alarm_second1 = 0;
              tft.fillRect(x - 360, y, 265, 65, background);
              draw_alarm_time_sm(x - 360, y, color);
          }
          else
          {
              
              timer_hour10 = 0;
              timer_hour1 = 0;
              timer_minute10 = 0;
              timer_minute1 = 0;
              timer_second10 = 0;
              timer_second1 = 0;
              tft.fillRect(x - 360, y, 265, 65, background);
              draw_timer_time_sm(x - 360, y, color);
          }
      }
      // Is it a backspace?
      if (num == 12)
      {
          Serial.println("Backspece");
          if (alarm)
          {
              alarm_second1 = alarm_second10;
              alarm_second10 = alarm_minute1;
              alarm_minute1 = alarm_minute10;
              alarm_minute10 = alarm_hour1;
              alarm_hour1 = alarm_hour10;
              alarm_hour10 = 0;
              tft.fillRect(x - 360, y, 265, 65, background);
              draw_alarm_time_sm(x - 360, y, color);
          }
          else
          {
              timer_second1 = timer_second10;
              timer_second10 = timer_minute1;
              timer_minute1 = timer_minute10;
              timer_minute10 = timer_hour1;
              timer_hour1 = timer_hour10;
              timer_hour10 = 0;
              tft.fillRect(x - 360, y, 265, 65, background);
              draw_timer_time_sm(x - 360, y, color);
          }
      }

      tft.graphicsMode();
      if (alarm)
      {
          for (int j = 0; j < 3; j++)
          {
              int xd = touchx - (x - 352 + j * 125);
              int yd = touchy - (y + 270);
              Serial.print(xd);
              Serial.print("  ****  ");
              Serial.println(yd);
             
              if ((abs(xd) < 40) && (abs(yd < 40)))
              {
               //   tft.fillCircle(x - 352 + j * 125, y + 270, 40, RA8875_GREEN);
                  delay(500);
                  if (j == 0)     // Alarm on or off
                  {
                        if (alarm_on)
                        {
                            alarm_on = 0;
                        }
                        else
                        {
                            alarm_on = 1;
                            alarm_hour = alarm_hour10 * 10;
                            alarm_hour += alarm_hour1;
                            alarm_minute = alarm_minute10 * 10;
                            alarm_minute += alarm_minute1;
                        }
                        tft.fillRect(x - 90, y + 140, 50, 30, background);
                        tft.textMode();
                        tft.textTransparent(color);
                        tft.textEnlarge(1);
                        tft.textSetCursor(x - 90, y + 140);
                        tft.textWrite(off_on[alarm_on], 3);
                  }
                  if (j == 1)     // Alarm am or pm
                  {
                       
                        if (alarm_ampm == 1)
                        {
                          alarm_ampm = 0;
                        }
                        else
                        {
                          alarm_ampm = 1;
                        }
                        tft.fillRect(x - 90, y + 30, 30, 30, background);
                        tft.textMode();
                        tft.textTransparent(color);
                        tft.textEnlarge(1);
                        tft.textSetCursor(x - 90, y + 30);
                        tft.textWrite(am_pm[alarm_ampm], 2);
                  }
                  if (j == 2)     // Exit the alarm
                  {
                      mode = 0;
                      tft.fillScreen(background);
                      draw_time(true);
                      draw_date_temp_hum(true);
                      draw_weather(50, 370, RA8875_BLACK, "Home");
                      return;
                  }
              }
          }
      }
      else
      {
          for (int j = 0; j < 2; j++)
          {
              int xd = touchx - (x - 290 + j * 125);
              int yd = touchy - (y + 270);
             
              if ((abs(xd) < 40) && (abs(yd < 40)))
              {
                  tft.fillCircle(x - 290 + j * 125, y + 270, 40, RA8875_GREEN);
                  delay(500);
                  if (j == 0)
                  {
                      // Start the timer
                       if (!((timer_second1 == 0) &&
                          (timer_second10 == 0) &&
                          (timer_minute1 == 0) &&
                          (timer_minute10 == 0) &&
                          (timer_hour1 == 0) &&
                          (timer_hour10 == 0)))
                      {
                          mode = 1;
                          timer_done = false;
                          last_timer_hour10 = -1;
                          last_timer_hour1 = -1;
                          last_timer_minute10 = -1;
                          last_timer_minute1 = -1;
                          last_timer_second10 = -1;
                          last_timer_second1 = -1;
                          tft.fillScreen(background);
                          return;
                      }
                  }
                  if (j == 1)
                  { 
                      // Exit the timer
                      timer_done = true;
                  }
                  mode = 0;
                  tft.fillScreen(background);
                  draw_time(true);
                  draw_date_temp_hum(true);
                  draw_weather(50, 370, RA8875_BLACK, "Home");
                  return;
              }
          }
      }
  }
}

void mandelbrot()
{
    double xa[10] = {0.0, -0.74, -0.117, -1.985462, -1.228, -0.818, 0.261, -0.6, -0.7212, -1.448};
    double ya[10] = {0.0, 0.167, 1.019, 0.0, 0.324, 0.21, -0.001618, 0.64, 0.356, 0.0};
    double maga[10] = {0.5, 100.0, 6.54, 1000.0, 7.14, 34.0, 1115.0, 7.0, 700.0, 11.0};
    int t[10] = {5, 2, 4, 4, 4, 3, 0, 4, 1, 4};
    int palnum;
    bool flag = false;
    tft.graphicsMode();
    while (true)
    {
        for (int counter = 0; counter < 10; counter++)
        {
            double x = xa[counter];
            double y = ya[counter];
            double width = 480;
            double mag =  maga[counter];
            double side = 2.0 / mag;
            double ymax = y + side / 2.0;
            double xmin = x - side * 0.5;
            double gap = side / width;
            double bb = ymax + gap;
            int maxdwell = 511;
            tft.fillScreen(background);
            int j = 0;
            while ((j < width))
            {
               bb -= gap; double aa = xmin - gap;
               for (int i = 0; i < width; i++)
               {
                  aa += gap; double xx = 0; double yy = 0;
                  double x2 = 0; double y2 = 0;
                  int count;
                  // This computes the count (dwell) for each pixel
                  for (count = 0; count < maxdwell; count++)
                  {
                     yy = (xx + xx) * yy + bb; xx = x2 - y2 + aa;
                     if ((y2 = yy * yy) + (x2 = xx * xx) >= 4) break;
                  }
                 
                  // Now color the pixel black for inside the Mandelbrot set
                  if (count == maxdwell)
                  {
                      tft.drawPixel(i + 160, j, 0x0000);
                  }
                  else
                  {  // Color the pixel based on the count (dwell) and palette used
                     switch (t[counter]) // t is the different intensity of the palette
                     {
                       case 0: palnum = (count / 8) % 128;
                               break;
                       case 1: palnum = (count / 4) % 128;
                               break;
                       case 2: palnum = (count / 2) % 128;
                               break;
                       case 3: palnum = count % 128;
                               break;
                       case 4: palnum = (count % 64) * 2 + 1;
                               break;
                       case 5: palnum = (count % 32) * 4 + 3;
                               break;        
                     }
                     tft.drawPixel(i + 160, j, palette[palnum]);
                  }
                  if (get_touch(&touchx, &touchy)){flag = true; break;}
               }
               j++;
               if (flag) break;
            }
            if (flag) break;
            delay(60000);
        }
        if (flag) break;
    }
    mode = 0;
    tft.fillScreen(background);
    draw_time(true);
    draw_date_temp_hum(true);
    draw_weather(50, 370, RA8875_BLACK, "Home");
}

void create_pallette()
{
    unsigned long red = 0x001F, green = 0x003F, blue = 0x001F;
    unsigned long tempcolor;
    tft.graphicsMode();
    tft.fillScreen(background);
    for (int i = 0; i < 128; i++)
    {
         palette[i] = 0;
         palette[i] = red << 11;
         tempcolor = green << 5;
         palette[i] += tempcolor;
         palette[i] += blue;
//         Serial.print(i);
//         Serial.print("  ");
//         Serial.print(red);
//         Serial.print("  ");
//         Serial.print(green);
//         Serial.print("  ");
//         Serial.print(blue);
//         Serial.print("    ");
//         Serial.println(palette[i], HEX);
         if (i < 32)
         {
             // red and green go to 0 leaving blue
             red -= 1;
             green -= 2;
         }
         if ((i >= 32) && (i < 64))
         {
             // green goes up and blue goes down leaving green
             green += 2;
             blue -= 1;
         }
         if ((i >= 64) && (i < 96))
         {
             // red goes up leaving yellow
             red += 1;
         }
          if ((i >= 96) && (i < 128))
         {
             // grenn goes down leaving red
             green -= 2;
         }
         if (i == 31)  { red = 0; green = 0; }
         if (i == 63)  { blue = 0; green = 63;}
         if (i == 95) red = 31;
       
        // tft.fillRect(10 + i  * 4, 10, 4, 50, palette[i]);
    }
}

// ******************************************************************************
// Has serial data come in?
void check_incomming()
{
    digitalWrite(TEST, HIGH);
    bool flag = false, tflag = false;
    // Check for incomming command from ESP32
      mySerialEvent();
      // Has a new string arrived from ESP32
      if (myStringComplete)
      { 
        Serial.print(myInputString);
        Serial.println(" being parsed");
        // Is it a time command? 
        if (myInputString.substring(0,4) == "Time")
        {
             // Check if length is correct
             Serial.println(myInputString.length());
             if (myInputString.length() != 19) tflag = true; 
             // Serial.println(myInputString);
             // Parse out the hour, minute and second
             b1 = (byte(myInputString[4]) - 48) * 10;
             b2 = byte(myInputString[5]) - 48;
             htemp = b1 + b2;
             b1 = (byte(myInputString[6]) - 48) * 10;
             b2 = byte(myInputString[7]) - 48;
             mtemp = b1 + b2;
             b1 = (byte(myInputString[8]) - 48) * 10;
             b2 = byte(myInputString[9]) - 48;
             stemp = b1 + b2;
             // Parse out tear, month and day
             b1 = byte(myInputString[10]) - 48;
             b2 = byte(myInputString[11]) - 48;
             b3 = byte(myInputString[12]) - 48;
             b4 = byte(myInputString[13]) - 48;
             iyear = b1 * 1000 + b2 * 100 + b3 * 10 + b4;
             b1 = (byte(myInputString[14]) - 48) * 10;
             b2 = byte(myInputString[15]) - 48;
             bmonth = b1 + b2;
             b1 = (byte(myInputString[16]) - 48) * 10;
             b2 = byte(myInputString[17]) - 48;
             bdayOfMonth = b1 + b2;
             
             // We do lots of check to make sure time is in the correct format
             
             Serial.print(iyear);
             Serial.print("  ");
             if (!((iyear > 2018) && (iyear < 2100))) tflag = true; // Yikes another y21k :-)
             
             Serial.print(bmonth);
             Serial.print("  ");
             if (!((bmonth >= 1) && (bmonth <= 12))) tflag = true;
             
             Serial.print(bdayOfMonth);
             Serial.print("  ");
             if (!((bdayOfMonth >= 1) && (bdayOfMonth <= 31))) tflag = true;
             
             Serial.print(htemp);
             Serial.print("  ");
             if (!((htemp >= 0) && (htemp <= 23))) tflag = true;  // 24 hour format at this point
            
             Serial.print(mtemp);
             Serial.print("  ");
             if (!((mtemp >= 0) && (mtemp <= 59))) tflag = true;
            
             Serial.print(stemp);
             Serial.print(" - ");
              if (!((stemp >= 0) && (stemp <= 59))) tflag = true;
             // Reset the Real Time Clock
             if (!tflag)
             {
                 Serial.println("Time reset");
                 rtc.adjust(DateTime(iyear, bmonth, bdayOfMonth, htemp, mtemp, stemp));
                 delay(10);
             }
        }
       

        // Is it a weather command?
        if ((myInputString.substring(0,7) == "Weather") || (myInputString.substring(0,4) == "Home"))
        {
          // Parse out the weather description
          if (myInputString.substring(0,4) == "Home") flag = true;
          
          String wtemp;
          int i0 = myInputString.indexOf(',');
          Serial.println(i0);
          int i1 = myInputString.indexOf(',', i0 + 1);
          wtemp = myInputString.substring(i0 + 1, i1);
          wtemp.setCharAt(0, wtemp.charAt(0) - 0x20);
          if (flag)
          {
              wtemp.toCharArray(hidesc, 25);
          }
          else
          {
              wtemp.toCharArray(idesc, 25);
          }
          if (debug) Serial.println(wtemp);
          
          // Parse out the temperature
          int index = i1 + 1;
          int i2 = myInputString.indexOf(',', index);
          wtemp = myInputString.substring(i1 + 1, i2);
          tlen = wtemp.length();
           if (flag)
          {
              wtemp.toCharArray(hitempfar, tlen + 1);
              for (int i = 0; i < 6; i++)
              {
                  if (hitempfar[i] != lasttempfar[i])
                  {
                     String lt = String(bhour);
                     lt += ":";
                     if (now_time.minute() < 10) lt += "0";
                     lt += String(now_time.minute());
                     lt.toCharArray(last_time, 6);
                   //  Serial.println(lt);
                   //  Serial.println(last_time);
                     for (int i = 0; i <6; i++)
                     {
                         lasttempfar[i] = hitempfar[i];
                     }
                     break;
                  }
              }
          }
          else
          {
              wtemp.toCharArray(itempfar, tlen + 1);
          }
          if (debug) Serial.println(wtemp);
          
          // Parse out the humidity
          i1 = i2;
          index = i1 + 1;
          i2 = myInputString.indexOf(',', index);
          wtemp = myInputString.substring(i1 + 1, i2);
          hlen = wtemp.length();
          if (flag)
          {
              wtemp.toCharArray(hihumid, hlen + 1);
          }
          else
          {
              wtemp.toCharArray(ihumid, hlen + 1);
          }
          if (debug) Serial.println(wtemp);

          // Parse out the wind speed
          i1 = i2;
          index = i1 + 1;
          i2 = myInputString.indexOf(',', index);
          wtemp = myInputString.substring(i1 + 1, i2);
          wlen = wtemp.length();
          if (flag)
          {
             wtemp.toCharArray(hiwind, wlen + 1);
          }
          else
          {
             wtemp.toCharArray(iwind, wlen + 1);
          }
           if (debug) Serial.println(wtemp);
          // Parse out the wind direction
          i1 = i2;
          index = i1 + 1;
          i2 = myInputString.indexOf(',', index);
          wtemp = myInputString.substring(i1 + 1, i2);
          float wf = wtemp.toFloat();
          if (flag)
          {
              if (wf < 22.5) hwi = 0;
              if ((wf >= 22.5) && (wf < 67.5)) hwi = 1;
              if ((wf >= 67.5) && (wf < 112.5)) hwi = 2;
              if ((wf >= 112.5) && (wf < 157.5)) hwi = 3;
              if ((wf >= 157.5) && (wf < 202.5)) hwi = 4;
              if ((wf >= 202.5) && (wf < 247.5)) hwi = 5;
              if ((wf >= 247.5) && (wf < 292.5)) hwi = 6;
              if ((wf >= 292.5) && (wf < 337.5)) hwi = 7;
              if (wf >= 337.5) hwi = 0;
          }
          else
          {
              if (wf < 22.5) wi = 0;
              if ((wf >= 22.5) && (wf < 67.5)) wi = 1;
              if ((wf >= 67.5) && (wf < 112.5)) wi = 2;
              if ((wf >= 112.5) && (wf < 157.5)) wi = 3;
              if ((wf >= 157.5) && (wf < 202.5)) wi = 4;
              if ((wf >= 202.5) && (wf < 247.5)) wi = 5;
              if ((wf >= 247.5) && (wf < 292.5)) wi = 6;
              if ((wf >= 292.5) && (wf < 337.5)) wi = 7;
              if (wf >= 337.5) wi = 0;
          }
          if (debug) Serial.println(wtemp);
          
          // Parse out the weather location
          i1 = i2;   
          wtemp = myInputString.substring(i1 + 1, myInputString.length() - 1);
          if (flag)
          {
               wtemp.toCharArray(hiname, 25);
               Serial.println(wtemp);
          }
          else
          {
              wtemp.toCharArray(iname, 25);
          }
          if (debug) Serial.println(wtemp);
          // Reset our values for the next incoming string
          flag = false;
        }
      
      // Is it a stock quote?
      if (myInputString.substring(0, 5) == "Stock")
      {
        Serial.println(myInputString);
        //Stock,MSFT,102.305,Microsoft Corporation
        String stemp;
        int i0 = myInputString.indexOf(',');
        int i1 = myInputString.indexOf(',', i0 + 1);
        stemp = myInputString.substring(i0 + 1, i1);
        for (int i = 0; i < 7; i++) stock_symbol[i] = ' ';
        stemp.toCharArray(stock_symbol, 8);
        int i2 = myInputString.indexOf(',', i1 + 1);
        stemp = myInputString.substring(i1 + 1, i2);
        for (int i = 0; i < 9; i++) stock_price[i] = ' ';
        stemp.toCharArray(stock_price, 10);
        stemp = myInputString.substring(i2 + 1, myInputString.length() - 1);
       for (int i = 0; i < 29; i++) stock_name[i] = ' ';
        stemp.toCharArray(stock_name, 30);
        
      }
       
   }   
      myStringComplete = false;
      myInputString = ""; 
      digitalWrite(TEST, LOW);
}

// *********************************************************************************
// Detect and store an incoming serial string
void mySerialEvent()
{
  while ((Serial3.available() > 0) && (myStringComplete == false))
  {
    // get the new byte:
    char inChar = (char)Serial3.read();
    // if the incoming character is a newline, set a flag
    // so the main loop can do something about it:
    // This changed so no \n at end
    if (inChar == '\n')
    {
      myStringComplete = true;
      break;
    }
    // add it to the inputString:
    myInputString += inChar;
  
    delay(2);
  }
  if (debug_min)
  {
      if (myInputString.length() > 0)
      {
          Serial.print("Input string = ");
          Serial.println(myInputString);
      }
//      Serial.print("  ");
//      Serial.println(myStringComplete);
  }
}

// *****************************************************************************
// Display a bmp file
bool bmpDraw(char *filename, int x, int y) {
  File     bmpFile;
  int      bmpWidth, bmpHeight;   // W+H in pixels
  uint8_t  bmpDepth;              // Bit depth (currently must be 24)
  uint32_t bmpImageoffset;        // Start of image data in file
  uint32_t rowSize;               // Not always = bmpWidth; may have padding
  uint8_t  sdbuffer[3*BUFFPIXEL]; // pixel in buffer (R+G+B per pixel)
  uint16_t lcdbuffer[BUFFPIXEL];  // pixel out buffer (16-bit per pixel)
  uint8_t  buffidx = sizeof(sdbuffer); // Current position in sdbuffer
  boolean  goodBmp = false;       // Set to true on valid header parse
  boolean  flip    = true;        // BMP is stored bottom-to-top
  int      w, h, row, col;
  uint8_t  r, g, b;
  uint32_t pos = 0, startTime = millis();
  uint8_t  lcdidx = 0;
  boolean  first = true;

  if((x >= tft.width()) || (y >= tft.height())) return false;

  Serial.println();
  Serial.print(F("Loading image '"));
  Serial.print(filename);
  Serial.println('\'');

  // Open requested file on SD card
  if ((bmpFile = SD.open(filename)) == NULL) {
    Serial.println(F("File not found"));
    return false;
  }

  // Parse BMP header
  if(read16(bmpFile) == 0x4D42) { // BMP signature
    Serial.println(F("File size: ")); 
    Serial.println(read32(bmpFile));
    (void)read32(bmpFile); // Read & ignore creator bytes
    bmpImageoffset = read32(bmpFile); // Start of image data
    Serial.print(F("Image Offset: ")); 
    Serial.println(bmpImageoffset, DEC);

    // Read DIB header
    Serial.print(F("Header size: ")); 
    Serial.println(read32(bmpFile));
    bmpWidth  = read32(bmpFile);
    bmpHeight = read32(bmpFile);

    if(read16(bmpFile) == 1) { // # planes -- must be '1'
      bmpDepth = read16(bmpFile); // bits per pixel
      Serial.print(F("Bit Depth: ")); 
      Serial.println(bmpDepth);
      if((bmpDepth == 24) && (read32(bmpFile) == 0)) { // 0 = uncompressed
        goodBmp = true; // Supported BMP format -- proceed!
        Serial.print(F("Image size: "));
        Serial.print(bmpWidth);
        Serial.print('x');
        Serial.println(bmpHeight);

        // BMP rows are padded (if needed) to 4-byte boundary
        rowSize = (bmpWidth * 3 + 3) & ~3;

        // If bmpHeight is negative, image is in top-down order.
        // This is not canon but has been observed in the wild.
        if(bmpHeight < 0) {
          bmpHeight = -bmpHeight;
          flip      = false;
        }

        // Crop area to be loaded
        w = bmpWidth;
        h = bmpHeight;
        if((x+w-1) >= tft.width())  w = tft.width()  - x;
        if((y+h-1) >= tft.height()) h = tft.height() - y;

        // Set TFT address window to clipped image bounds

        for (row=0; row<h; row++) { // For each scanline...
          // Seek to start of scan line.  It might seem labor-
          // intensive to be doing this on every line, but this
          // method covers a lot of gritty details like cropping
          // and scanline padding.  Also, the seek only takes
          // place if the file position actually needs to change
          // (avoids a lot of cluster math in SD library).
          if(flip) // Bitmap is stored bottom-to-top order (normal BMP)
            pos = bmpImageoffset + (bmpHeight - 1 - row) * rowSize;
          else     // Bitmap is stored top-to-bottom
          pos = bmpImageoffset + row * rowSize;
          if(bmpFile.position() != pos) { // Need seek?
            bmpFile.seek(pos);
            buffidx = sizeof(sdbuffer); // Force buffer reload
          }

          for (col=0; col<w; col++) { // For each column...
            // Time to read more pixel data?
            if (buffidx >= sizeof(sdbuffer)) { // Indeed
              // Push LCD buffer to the display first
              if(lcdidx > 0) {
                tft.drawPixel(col+x, row+y, lcdbuffer[lcdidx]);
                lcdidx = 0;
                first  = false;
              }

              bmpFile.read(sdbuffer, sizeof(sdbuffer));
              buffidx = 0; // Set index to beginning
            }

            // Convert pixel from BMP to TFT format
            b = sdbuffer[buffidx++];
            g = sdbuffer[buffidx++];
            r = sdbuffer[buffidx++];
            lcdbuffer[lcdidx] = color565(r,g,b);
            tft.drawPixel(col+x, row+y, lcdbuffer[lcdidx]);
          } // end pixel
        // Need to check for a touch to interupt drawing
          if (get_touch(&touchx, &touchy))
          {
            mode = 0;
            tft.fillScreen(background);
            draw_time(true);
            draw_date_temp_hum(true);
            draw_weather(50, 350, RA8875_BLACK, "Home");
            return true;
          }
        } // end scanline

        // Write any remaining data to LCD
        if(lcdidx > 0) {
          tft.drawPixel(col+x, row+y, lcdbuffer[lcdidx]);
        } 

        Serial.print(F("Loaded in "));
        Serial.print(millis() - startTime);
        Serial.println(" ms");

      } // end goodBmp
    }
  }

  bmpFile.close();
  if(!goodBmp) Serial.println(F("BMP format not recognized."));
  return false;
}

// **********************************************************************************
// These read 16- and 32-bit types from the SD card file.
// BMP data is stored little-endian, Arduino is little-endian too.
// May need to reverse subscript order if porting elsewhere.

uint16_t read16(File f) {
  uint16_t result;
  ((uint8_t *)&result)[0] = f.read(); // LSB
  ((uint8_t *)&result)[1] = f.read(); // MSB
  return result;
}

// **********************************************************************************
uint32_t read32(File f) {
  uint32_t result;
  ((uint8_t *)&result)[0] = f.read(); // LSB
  ((uint8_t *)&result)[1] = f.read();
  ((uint8_t *)&result)[2] = f.read();
  ((uint8_t *)&result)[3] = f.read(); // MSB
  return result;
}

// **********************************************************************************
uint16_t color565(uint8_t r, uint8_t g, uint8_t b) {
  return ((r & 0xF8) << 8) | ((g & 0xFC) << 3) | (b >> 3);
}

// **********************************************************************************
byte decToBcd(byte val){
  // Convert normal decimal numbers to binary coded decimal
  return ( (val/10*16) + (val%10) );
}

// **********************************************************************************
void printDirectory(File dir) {
   while(true) {
     
     File entry =  dir.openNextFile();
     if (! entry) {
       // no more files
       //Serial.println("**nomorefiles**");
       break;
     }
     if (!entry.isDirectory())
     {
        Serial.println(entry.name());
        tft.fillScreen(background);
        if (bmpDraw(entry.name(), 0, 0)) return;
        delay(10000);
     }
     entry.close();
   }
}

// *********************************************************************************
// Radio Menu
void radio_menu(int x, int y, unsigned long color)
{
  char tm[6] = "Radio";
  char radio_on[3] = "On";
  char radio_off[4] = "Off";  
  char b[3][8] = {"On Off", "Station", " Exit"};
  bool flag = false;
  int local_mode;

  tft.textMode();
  tft.textEnlarge(3);
  tft.textTransparent(color);
  tft.textSetCursor(x + 10, y);
  tft.textWrite(tm, 5);
  tft.textSetCursor(x + 30, y + 275);
  if (my_radio_on)
  {
      tft.textWrite(radio_on, 2);
  }
  else
  {
      tft.textWrite(radio_off, 3);
  }
  // Write labels on larger buttons
  tft.textEnlarge(1);
  // Draw button names
  for (int ii = 0; ii < 3; ii++)
  {
      tft.textSetCursor(x - 90 + ii * 125, y + 205);
      tft.textWrite(b[ii], 7);
      tft.textSetCursor(x - 89 + ii * 125, y + 205);
      tft.textWrite(b[ii], 7);
  }
  // Draw circles for circular buttons
  tft.graphicsMode();
  for (int i = 0; i < 3; i++)
  {
      for (int k = 0; k < 4; k++)
      {
          tft.drawCircle(x - 42 + i * 125, y + 160, 40 - k, color);
      }
  } 

  while (!flag)
  {
      // Now wait for touch
      while(!get_touch(&touchx, &touchy)){}
      
      for (int j = 0; j < 3; j++)
      {
          int xd = touchx - (x - 42 + j * 125);
          int yd = touchy - (y + 160);
//              Serial.print(xd);
//              Serial.print("  xd  yd  ");
//              Serial.println(yd);
          if ((abs(xd) < 40) && (abs(yd) < 40))
          {
              tft.fillCircle((x - 42 + j * 125), (y + 160), 36, RA8875_GREEN);
              delay(200);
              tft.fillCircle((x - 42 + j * 125), (y + 160), 36, background);
              local_mode = j;
              Serial.print("Local mode is =  ");
              Serial.println(local_mode);
              delay(200);
              if (local_mode == 0)    // Turn radio on or off
              {
                  if (!my_radio_on)
                  {
                      Serial3.println("Radio\n");
                      Serial.println("Radio turned on");
                      tft.fillRect(x + 30, y + 275, 100, 75, background);
                      tft.textMode();
                      tft.textEnlarge(3);
                      tft.textTransparent(color);
                      tft.textSetCursor(x + 30, y + 275);
                      tft.textWrite(radio_on, 2);
                      my_radio_on = true;
                      break;
                  }
                  else
                  {
                      // Throw an interrupt on ESP32 to turn radio off
                      digitalWrite(STOP_RADIO_PIN, LOW);
                      delay(10);
                      digitalWrite(STOP_RADIO_PIN, HIGH);
                      Serial.println("Radio turned off");
                      tft.fillRect(x + 30, y + 275, 100, 75, background);
                      tft.textMode();
                      tft.textEnlarge(3);
                      tft.textTransparent(color);
                      tft.textSetCursor(x + 30, y + 275);
                      tft.textWrite(radio_off, 3);
                      my_radio_on = false;
                      break;
                  }
              }
              if (local_mode == 1)    // Change the station
              {
                  // Throw an interrupt on ESP32 to stop radio
                  digitalWrite(STOP_RADIO_PIN, LOW);
                  delay(10);
                  digitalWrite(STOP_RADIO_PIN, HIGH);
                  // Throw an interrupt on ESP32 to change station
                  digitalWrite(NEXT_STATION_PIN, LOW);
                  delay(10);
                  digitalWrite(NEXT_STATION_PIN, HIGH);
                  // Restart radio
                  Serial3.println("Radio\n");
                  break;
              }
              if (local_mode == 2);   // Leave radio menu
              {
                  flag = true;
                  delay(200);
              }
          }
      }
  }

  mode = 0;
  tft.fillScreen(background);
  draw_time(true);
  draw_date_temp_hum(true);
  draw_weather(50, 370, RA8875_BLACK, "Home");
}
